/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* $Id: $ */
package org.apache.fop.fonts.autodetect;

import java.io.File;
import java.io.FileFilter;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.fop.apps.FOPException;
import org.apache.fop.fonts.FontResolver;
import org.apache.fop.util.FileUtil;
import org.apache.fop.util.LogUtil;

/**
 * 
 * Helps to autodetect/locate available operating system fonts
 * 
 * @author Adrian Cumiskey
 */
public class FontFileFinder {
    /** logging instance */
    private static final Log log = LogFactory.getLog(FontFileFinder.class);

    /** font file filter */
    private static FileFilter fontFileFilter = null;

    /** font directory filter */
    private static FileFilter fontDirectoryFilter = null;
    
    /**
     * Directory filter used to find valid font directories
     * @return font file filter
     */
    private static FileFilter getFontDirectoryFilter() {
        if (fontDirectoryFilter == null) {
            fontDirectoryFilter = new FontDirFilter();
        }
        return fontDirectoryFilter;
    }

    /**
     * File filter used to find font files
     * @return font file filter
     */
    private static FileFilter getFontFileFilter() {
        if (fontFileFilter == null) {
            fontFileFilter = new FontFileFilter();
        }
        return fontFileFilter;
    }

    /**
     * Finds/identifies font files within a given directory.
     * 
     * @param fontDir File object representing fonts location
     * @param fontResolver font resolver used to resolve font
     * @param recursive look in sub directories for fonts
     * @param strict validate configuration strictly
     * @return list of found font files
     * @throws FOPException if something's wrong with the config data
     */
    public static List find(
            File fontDir, FontResolver fontResolver, boolean recursive, boolean strict)
            throws FOPException {

        List fontFileList = new java.util.ArrayList();
        
        if (!FileUtil.validDirectory(fontDir, log)) {
            LogUtil.handleError(log, "Fonts configuration directory '"
                            + fontDir.getAbsolutePath() + "' is not valid", strict);
            return fontFileList;
        }
                       
        File[] fontFiles = fontDir.listFiles(getFontFileFilter());
        log.info(fontDir + ": found " + fontFiles.length + " font"
                + ((fontFiles.length == 1) ? "" : "s"));
        for (int i = 0; i < fontFiles.length; i++) {
            File fontFile = fontFiles[i];
            fontFileList.add(fontFile);
        }

        // examine sub directories
        if (recursive) {
            File[] fontSubdirList = fontDir.listFiles(getFontDirectoryFilter());
            for (int i = 0; i < fontSubdirList.length; i++) {
                List subDirFontFileList = find(fontSubdirList[i], fontResolver, recursive, strict);
                fontFileList.addAll(subDirFontFileList);
            }
        }        
        return fontFileList;
    }    
}